#include <iostream>
#include "dlist.h"   
#include <functional>
#include <map>
#include <cassert>

using namespace std;

typedef int (*TestCase)();

int case0();
int case1();
int case2();
int case3();
int case4();
int case5();
int case6();

map<int, TestCase> cases = {
        {0, case0},
        {1, case1},
        {2, case2},
        {3, case3},
        {4, case4},
        {5, case5},
        {6, case6}
};

const int CaseNum = 7;

int main() {
    int testnum;
    cin >> testnum;
    int i = cases[testnum % CaseNum]();
    cout << "Test success." << endl;
}

int case0() {
    double* d1 = new double(1.0);
    double* d2 = new double(2.0);

    Dlist<double> list;
    assert(list.isEmpty());
    list.insertFront(d1);
    assert(!list.isEmpty());
    list.insertFront(d2);
    assert(!list.isEmpty());

    double* r2 = list.removeBack();
    assert(!list.isEmpty());
    double* r1 = list.removeBack();
    assert(list.isEmpty());

    assert(r1 == d2 && *r1 == *d2);
    assert(r2 == d1 && *r2 == *d1);

    delete d1;
    delete d2;

    return 0;
}

int case1() {
    double* d1 = new double(1.0);
    double* d2 = new double(2.0);

    Dlist<double> list;
    assert(list.isEmpty());
    list.insertBack(d1);
    assert(!list.isEmpty());
    list.insertBack(d2);
    assert(!list.isEmpty());

    double* r2 = list.removeFront();
    assert(!list.isEmpty());
    double* r1 = list.removeFront();
    assert(list.isEmpty());

    assert(r1 == d2 && *r1 == *d2);
    assert(r2 == d1 && *r2 == *d1);

    delete d1;
    delete d2;

    return 0;
}

int case2() {
    int* i1 = new int(1);
    int* i2 = new int(2);
    int* i3 = new int(3);

    Dlist<int> list;
    list.insertFront(i1);
    list.insertBack(i3);
    list.insertFront(i2);

    int* di1 = list.removeBack();
    assert(di1 == i3 && *di1 == 3);
    int* di2 = list.removeBack();
    assert(di2 == i1 && *di2 == 1);
    int* di3 = list.removeFront();
    assert(di3 == i2 && *di3 == 2);
    assert(list.isEmpty());

    delete i1;
    delete i2;
    delete i3;

    return 0;
}

int case3() {
    int* i1 = new int(1);
    int* i2 = new int(2);
    int* i3 = new int(3);

    Dlist<int> list1;
    list1.insertBack(i3);
    list1.insertBack(i2);
    list1.insertBack(i1);

    Dlist<int> list2(list1);
    int* r1 = list2.removeFront();
    assert(*r1 == 3);
    *r1 = 4;
    int* r2 = list2.removeBack();
    assert(*r2 == 1);
    *r2 = 2;
    int* r3 = list2.removeBack();
    assert(*r3 == 2);
    *r3 = 5;
    assert(list2.isEmpty());

    delete r1;
    delete r2;
    delete r3;

    assert(!list1.isEmpty());
    r1 = list1.removeBack();
    r2 = list1.removeFront();
    r3 = list1.removeBack();
    assert(list1.isEmpty());

    assert(*r1 == 1);
    assert(*r2 == 3);
    assert(*r3 == 2);

    delete r1;
    delete r2;
    delete r3;

    return 0;
}

int case4() {
    int* i1 = new int(1);
    int* i2 = new int(2);
    int* i3 = new int(3);

    Dlist<int> list1;
    list1.insertBack(i3);
    list1.insertBack(i2);
    list1.insertBack(i1);

    Dlist<int> list2 = list1;
    int* r1 = list2.removeFront();
    assert(*r1 == 3);
    *r1 = 4;
    int* r2 = list2.removeBack();
    assert(*r2 == 1);
    *r2 = 2;
    int* r3 = list2.removeBack();
    assert(*r3 == 2);
    *r3 = 5;
    assert(list2.isEmpty());

    delete r1;
    delete r2;
    delete r3;

    assert(!list1.isEmpty());
    r1 = list1.removeBack();
    r2 = list1.removeFront();
    r3 = list1.removeBack();
    assert(list1.isEmpty());

    assert(*r1 == 1);
    assert(*r2 == 3);
    assert(*r3 == 2);

    delete r1;
    delete r2;
    delete r3;

    return 0;
}

int case5() {
    int* i1 = new int(1);
    int* i2 = new int(2);
    int* i3 = new int(3);

    Dlist<int> list1;
    list1.insertBack(i3);
    list1.insertBack(i2);
    list1.insertBack(i1);

    Dlist<int> list2(list1);
    list2.insertFront(new int(4));
    list2.insertFront(new int(5));
    list1 = list2;

    int* r1 = list2.removeFront(); assert(*r1 == 5); *r1 = 1;
    int* r2 = list2.removeBack();  assert(*r2 == 1); *r2 = 7;
    int* r3 = list2.removeBack();  assert(*r3 == 2); *r3 = 6;
    assert(!list2.isEmpty());

    delete r1; delete r2; delete r3;

    r1 = list1.removeFront(); assert(*r1 == 5); *r1 = 1;
    r2 = list1.removeBack();  assert(*r2 == 1); *r2 = 7;
    r3 = list1.removeBack();  assert(*r3 == 2); *r3 = 6;
    assert(!list1.isEmpty());

    delete r1; delete r2; delete r3;

    return 0;
}

int case6() {
    int* i1 = new int(1);
    int* i2 = new int(2);
    int* i3 = new int(3);

    Dlist<int> list1;
    list1.insertBack(i3);
    list1.insertBack(i2);
    list1.insertBack(i1);

    Dlist<int> list2 = list1;
    list2.insertFront(new int(4));
    list2.insertFront(new int(5));
    list1 = list2;

    int* r1 = list2.removeFront(); assert(*r1 == 5); *r1 = 1;
    int* r2 = list2.removeBack();  assert(*r2 == 1); *r2 = 7;
    int* r3 = list2.removeBack();  assert(*r3 == 2); *r3 = 6;
    assert(!list2.isEmpty());

    delete r1; delete r2; delete r3;

    r1 = list1.removeFront(); assert(*r1 == 5); *r1 = 1;
    r2 = list1.removeBack();  assert(*r2 == 1); *r2 = 7;
    r3 = list1.removeBack();  assert(*r3 == 2); *r3 = 6;
    assert(!list1.isEmpty());

    delete r1; delete r2; delete r3;

    return 0;
}