#include <iostream>
#include <string>
#include <cstdlib>
#include "dlist.h"
using namespace std;

template <class T>
class Stack
{
    // OVERVIEW: a stack class, implemented based on Dlist
    Dlist<T> list;
public:
    bool isEmpty();
    // EFFECTS: returns true if stack is empty, false otherwise.

    void push(T *op);
    // MODIFIES: this
    // EFFECTS: adds op to the top of the stack.

    T *pop();
    // MODIFIES: this
    // EFFECTS: removes and returns the top of the stack.
    //          Throws an instance of emptyList if empty.

    T topValue();
    // EFFECTS: returns the value of the top of the stack.
    //          Throws an instance of emptyList if empty.

    void clear();
    // EFFECTS: returns an empty stack;

    Stack(const Dlist<T> &l = Dlist<T>()); // constructor with default
                                           // argument
};

template <class T>
bool hasAtLeastTwo(Stack<T> &stack);
// EFFECTS: returns true if the stack has at least two elements, false
//          otherwise.

template <class T>
bool calcTopTwo(Stack<T> &stack, T (*f)(T, T));
// MODIFIES: stack and cout.
// EFFECTS: if the stack has at least two elements, pops the top element op1
//          and the second to top element op2.  Then pushes the value of
//          f(op2, op1) into the stack and returns true. If the stack
//          does not have at least two elements, prints the message "Not
//          enough operands" and returns false.

/*
template <class T>
bool calcTopOne(Stack<T> &stack, T (*f)(T));
// MODIFIES: stack and cout.
// EFFECTS: if the stack is not empty, pops the top element op. Then pushes
//          the value of f(op) into the stack and returns true. If the stack
//          is empty, prints the message "Not enough operands" and returns
//          false.
*/

template <class T>
T add(T a, T b);

template <class T>
T subtract(T a, T b);

template <class T>
T multiply(T a, T b);

template <class T>
T division(T a, T b);

void notEnoughOperands();
// MODIFIES: cout
// EFFECTS: print an error message of "Not enough operands".

bool isDigit(char c);
// EFFECTS: returns true if c is a digit character.

bool isInteger(const string &s);
// EFFECTS: returns true if s represents a number.

int main(int argc, char *argv[])
{
    Stack<int> stack; 
    string input;
    do
    {
        cin >> input;
        if(isInteger(input))
        {
            int *op = new int(atoi(input.c_str()));
            stack.push(op);
        }
        else if(input == "+")
        {
            calcTopTwo(stack, add);
        }
        else if(input == "-")
        {
            calcTopTwo(stack, subtract);
        }
        else if(input == "*")
        {
            calcTopTwo(stack, multiply);
        }
        else if(input == "/")
        {
            if(!hasAtLeastTwo(stack))
                notEnoughOperands();
            else if(stack.topValue() == 0)
                cout << "Divide by zero" << endl;
            else
                calcTopTwo(stack, division);
        }
        else if(input == "n")
        {
            if(stack.isEmpty())
                notEnoughOperands();
            else
            {
                int *op = stack.pop();
                *op *= -1;
                stack.push(op);
            }
        }
        else if(input == "d")
        {
            if(stack.isEmpty())
                notEnoughOperands();
            else
            {
                int val = stack.topValue();
                int *op = new int(val);
                stack.push(op);
            }
        }
        else if(input == "r")
        {
            if(!hasAtLeastTwo(stack))
                notEnoughOperands();
            else
            {
                int *op1 = stack.pop();
                int *op2 = stack.pop();
                stack.push(op1);
                stack.push(op2);
            }
        }
        else if(input == "p")
        {
            if(stack.isEmpty())
                notEnoughOperands();
            else
            {
                int val = stack.topValue();
                cout << val << endl;
            }
        }
        else if(input == "c")
        {
            stack.clear();
        }
        else if(input == "a")
        {
            Stack<int> stack_copy(stack);
            while(!stack_copy.isEmpty())
            {
                int *op = stack_copy.pop();
                cout << *op << " " << flush;
                delete op;
            }
            cout << endl;
        }
        else if(input == "q")
            break;
        else
            cout << "Bad input" << endl;
    }
    while(1);
    return 0;
}

template <class T>
bool Stack<T>::isEmpty()
{
    return list.isEmpty();
}

template <class T>
void Stack<T>::push(T *op)
{
    list.insertBack(op);
}

template <class T>
T *Stack<T>::pop()
{
    return list.removeBack();
}

template <class T>
T Stack<T>::topValue()
{
    T *op = pop();
    push(op);
    return *op;
}

template <class T>
void Stack<T>::clear()
{
    list = Dlist<T>();
}

template <class T>
Stack<T>::Stack(const Dlist<T> &l): list(l)
{
}

template <class T>
bool hasAtLeastTwo(Stack<T> &stack)
{
    if(stack.isEmpty())
        return false;

    T *op = stack.pop();
    if(stack.isEmpty())
    {
        stack.push(op);
        return false;
    }

    stack.push(op);
    return true;
}

template <class T>
bool calcTopTwo(Stack<T> &stack, T (*f)(T, T))
{
    if(!hasAtLeastTwo(stack))
    {
        notEnoughOperands();
        return false;
    }

    T *op1 = stack.pop();
    T *op2 = stack.pop();
    T *result = new T;
    *result = f(*op2, *op1);
    delete op1;
    delete op2;
    stack.push(result);
    return true;
}

template <class T>
T add(T a, T b)
{
    return a+b;
}

template <class T>
T subtract(T a, T b)
{
    return a-b;
}

template <class T>
T multiply(T a, T b)
{
    return a*b;
}

template <class T>
T division(T a, T b)
{
    return a/b;
}

void notEnoughOperands()
{
    cout << "Not enough operands" << endl;
}

bool isDigit(char c)
{
    return (c >= '0' && c <= '9') ;
}

bool isInteger(const string &s)
{
    unsigned int i;
    if(s.empty())
        return false;

    if(s.size() == 1)
    {
        if(!isDigit(s[0]))
            return false;
        else
            return true;
    }

    if(s[0] != '-' && !isDigit(s[0]))
        return false;

    for(i = 1; i < s.size(); i++)
    {
        if(!isDigit(s[i]))
            return false;
    }

    return true;
}
