#ifndef __DLIST_IMPL_H__
#define __DLIST_IMPL_H__

#include <cassert>

template <class T>
bool Dlist<T>::isEmpty() const
{
    return !first;
}

template <class T>
void Dlist<T>::insertFront(T *op)
{
	node *np = new node;
	np->op = op;
	np->next = first;
    np->prev = NULL;
    // Fix the invariant for the prev field of the node next to np
    if(np->next == NULL)
        last = np;
    else
        np->next->prev = np;
	first = np;
}

template <class T>
void Dlist<T>::insertBack(T *op)
{
	node *np = new node;
	np->op = op;
	np->next = NULL;
    np->prev = last;
    // Fix the invariant for the next field of the node prev to np
    if(np->prev == NULL)
        first = np;
    else
        np->prev->next = np;
	last = np;
}

template <class T>
T *Dlist<T>::removeFront()
{
    if(isEmpty())
    {
        emptyList e;
        throw e;
    }

    node *victim = first;
    T *result = first->op;

    first = victim->next;
    delete victim;

    if(first == NULL)
        last = NULL;
    else
        first->prev = NULL;

    return result;
}

template <class T>
T *Dlist<T>::removeBack()
{
    if(isEmpty())
    {
        emptyList e;
        throw e;
    }

    node *victim = last;
    T *result = last->op;

    last = victim->prev;
    delete victim;

    if(last == NULL)
        first = NULL;
    else
        last->next = NULL;

    return result;
}

/*
    void makeEmpty();
    // EFFECT: called by constructors/operator= to establish empty
    //         list invariant
    
template <class T>
void Dlist<T>::makeEmpty()
{
    first = last = NULL;
}
*/

template <class T>
void Dlist<T>::removeAll()
{
    while(!isEmpty())
    {
        T *op = removeFront();
        delete op;
    }
}

template <class T>
void Dlist<T>::copyAll(const Dlist &l)
{
    node *lnp = l.first;
    while(lnp)
    {
        T *op = new T(*lnp->op);
        insertBack(op);
        lnp = lnp->next;
    }
}

template <class T>
Dlist<T>::Dlist(): first(NULL), last(NULL)
{
}

template <class T>
Dlist<T>::Dlist(const Dlist &l): first(NULL), last(NULL)
{
    copyAll(l);
}

template <class T>
Dlist<T> &Dlist<T>::operator=(const Dlist &l)
{
    if(this != &l)
    {
        removeAll();
        assert(!first && !last);
        copyAll(l);
    }
    return *this;
}
template <class T>
Dlist<T>::~Dlist()
{
    removeAll();
}

#endif /* __DLIST_IMPL_H__ */
