#include <iostream>
#include <string>
#include <cstdlib>
#include "dlist.h"
using namespace std;

template <class T>
class Queue
{
    // OVERVIEW: a queue class, implemented based on Dlist
    Dlist<T> list;
public:
    bool isEmpty();
    // EFFECTS: returns true if queue is empty, false otherwise.

    void push(T *op);
    // MODIFIES: this
    // EFFECTS: adds op to the end of the queue.

    T *pop();
    // MODIFIES: this
    // EFFECTS: removes and returns the front of the queue.
    //          Throws an instance of emptyList if empty.

    T frontValue();
    // EFFECTS: returns the value of the first element of the queue.
    //          Throws an instance of emptyList if empty.

    void clear();
    // EFFECTS: returns an empty queue;

    Queue(const Dlist<T> &l = Dlist<T>()); // constructor with default
                                           // argument
};

enum Status {PLATINUM, GOLD, SILVER, REGULAR};

const string StatusNames[] = {"platinum", "gold", "silver", "regular"};

const int numStatus = REGULAR+1;

struct Event
{
    int timestamp;
    string name;
    Status status;
    int duration;
};

void readOneCall(Queue<Event> &call_queue);
// EFFECTS: read one event and store that event in the call queue.

bool hasCalls(Queue<Event> &call_queue, Queue<Event> waiting_queues[]);
// REQUIRES: waiting_queues is an array of four Queue<Event> objects
// EFFECTS: returns true if call_queue or any one of the four waiting_queues
//          is not empty. Returns false otherwise.

int main(int argc, char *argv[])
{
    Queue<Event> call_queue;
    Queue<Event> waiting_queues[numStatus];
    int numEvents, i, tick, finish_time;

    cin >> numEvents;
    for(i = 0; i < numEvents; i++)
        readOneCall(call_queue);

    tick = 0;
    finish_time = 0;
    while(hasCalls(call_queue, waiting_queues) || tick <= finish_time)
    {
        cout << "Starting tick #" << tick << endl;
        while(!call_queue.isEmpty()
            && call_queue.frontValue().timestamp == tick)
        {
            Event *ep = call_queue.pop();
            waiting_queues[ep->status].push(ep);
            cout << "Call from " << ep->name << " a "
                << StatusNames[ep->status] << " member" << endl; 
        }
        if(tick >= finish_time)
        // Can take new calls
        {
            Event *ep = NULL;
            for(i = PLATINUM; i <= REGULAR; i++)
            {
                if(!waiting_queues[i].isEmpty())
                {
                    ep = waiting_queues[i].pop();
                    break;
                }
            }
            if(ep)
            {
                cout << "Answering call from " << ep->name << endl;
                finish_time = tick + ep->duration;
                delete ep;
            }
        }
        tick++;
    }
    return 0;
}

template <class T>
bool Queue<T>::isEmpty()
{
    return list.isEmpty();
}

template <class T>
void Queue<T>::push(T *op)
{
    list.insertBack(op);
}

template <class T>
T *Queue<T>::pop()
{
    return list.removeFront();
}

template <class T>
T Queue<T>::frontValue()
{
    T *op = pop();
    list.insertFront(op);
    return *op;
}

template <class T>
void Queue<T>::clear()
{
    list = Dlist<T>();
}

template <class T>
Queue<T>::Queue(const Dlist<T> &l): list(l)
{
}

void readOneCall(Queue<Event> &call_queue)
{
    Event *event = new Event;
    string status;

    cin >> (event->timestamp) >> (event->name)
        >> status >> (event->duration); 

    for(int i = PLATINUM; i <= REGULAR; i++)
    {
        if(status == StatusNames[i])
        {
            event->status = static_cast<Status>(i);
            call_queue.push(event);
        }
    }
}

bool hasCalls(Queue<Event> &call_queue, Queue<Event> waiting_queues[])
{
    if(!call_queue.isEmpty())
        return true;
    for(int i = PLATINUM; i <= REGULAR; i++)
    {
        if(!waiting_queues[i].isEmpty())
            return true;
    }
    return false;
}
